import type React from "react";
import type { JCRNodeWrapper } from "org.jahia.services.content";
/**
 * Generates an absolute area in which editors may insert content objects.
 *
 * @returns The AbsoluteArea component
 */
export declare function AbsoluteArea({ name, parent, areaView, view, allowedTypes, allowedNodeTypes, numberOfItems, readOnly, areaType, nodeType, parameters, }: Readonly<{
    /** The name of the area. */
    name: string;
    /**
     * Parent node where the area is stored in the JCR. Can be anywhere in the JCR. The parent node
     * must exist.
     */
    parent: JCRNodeWrapper;
    /**
     * The view to use for the area.
     *
     * @deprecated Use {@link #view} instead
     */
    areaView?: string;
    /** The view to use for the area. */
    view?: string;
    /**
     * The allowed types for the area.
     *
     * @deprecated Use {@link #allowedNodeTypes} instead
     */
    allowedTypes?: string[];
    /** The allowed types for the area. */
    allowedNodeTypes?: string[];
    /** The number of items to display in the area. */
    numberOfItems?: number;
    /**
     * Makes the area read-only.
     *
     * - When set to `false` (default), the area is editable on all pages.
     * - When set to `true`, the area is read-only on all pages.
     * - When set to `"children"`, the area is read-only on all pages except the one containing its
     *   node.
     *
     * @default false
     */
    readOnly?: boolean | "children";
    /**
     * Node type to be used to create the area (if the node does not exist yet)
     *
     * @deprecated Use {@link #nodeType} instead
     * @default jnt:contentList
     */
    areaType?: string;
    /**
     * Node type to be used to create the area (if the node does not exist yet)
     *
     * @default jnt:contentList
     */
    nodeType?: string;
    /**
     * Map of custom parameters that can be passed to the backend engine for advanced logic.
     *
     * @deprecated Not recommended
     */
    parameters?: Record<string, unknown>;
}>): React.JSX.Element;
