/**
 * Will render the given React component server side and hydrate it in the browser to make it
 * dynamic. Be careful, the component will not have access to the
 * '@jahia/javascript-modules-library' library from the browser.
 *
 * @returns The component to be hydrated in the browser
 */
export declare function HydrateInBrowser<T, U>(props: Readonly<{
    /** The React component. */
    child: React.ComponentType<{
        children: U;
    } & T>;
    /**
     * The React component props, these props will be serialized/deserialized to be usable server
     * and client side. The serialization is done using
     * [devalue](https://www.npmjs.com/package/devalue) allowing most standard JS types, including
     * `Set` and `Map`.
     */
    props: Omit<T & React.JSX.IntrinsicAttributes, "children">;
    children: U;
}>): React.JSX.Element;
export declare function HydrateInBrowser<U>(props: Readonly<{
    /** The React component. */
    child: React.ComponentType<{
        children: U;
    }>;
    children: U;
}>): React.JSX.Element;
export declare function HydrateInBrowser<T>(props: Readonly<{
    /** The React component. */
    child: React.ComponentType<T>;
    /**
     * The React component props, these props will be serialized/deserialized to be usable server
     * and client side. The serialization is done using
     * [devalue](https://www.npmjs.com/package/devalue) allowing most standard JS types, including
     * `Set` and `Map`.
     */
    props: T & React.JSX.IntrinsicAttributes;
}>): React.JSX.Element;
export declare function HydrateInBrowser(props: Readonly<{
    /** The React component. */
    child: React.ComponentType;
}>): React.JSX.Element;
