import type { JSX } from "react";
import type { JCRNodeWrapper } from "org.jahia.services.content";
/** Renders the children of the current node, and "Add content" buttons afterwards. */
export declare function RenderChildren({ view, pagination, filter, }: {
    /** View to use when rendering the children. */
    view?: string | undefined;
    /**
     * Pagination parameters:
     *
     * - `{ count: number; start?: number }` to specify the number of children to display and the
     *   starting index (defaults to 0).
     * - `{ count: number; page: number }` to specify the number of children to display and the page
     *   number.
     *
     * If not provided, all children will be displayed.
     */
    pagination?: {
        count: number;
        start?: number;
    } | {
        count: number;
        page: number;
    };
    /**
     * Filter to apply to the children:
     *
     * - A string to filter by node type.
     * - A function to filter by custom logic.
     *
     * @default "jnt:content"
     */
    filter?: string | ((node: JCRNodeWrapper) => boolean);
}): JSX.Element;
