declare module 'org.jahia.services.content.decorator' {
import { Locale, Set, List } from 'java.util';
import { JCRNodeWrapper, JCRNodeIteratorWrapper } from 'org.jahia.services.content';
/**
 * JCR node representing the Jahia virtual site.
 *
 * User: toto
 * Date: Mar 30, 2010
 * Time: 12:37:45 PM
*/
export interface JCRSiteNode extends JCRNodeWrapper {
  /**
   * @return list of inactive live languages
  */
  getInactiveLiveLanguages(): Set<string>;
  /**
   * @return list of inactive languages
  */
  getInactiveLanguages(): Set<string>;
  /**
   * @deprecated use either {@link #getActiveLiveLanguagesAsLocales} or {@link #getActiveLiveLanguages()} methods instead
  */
  getActiveLanguagesAsLocales(): Locale[];
  /**
   * @deprecated use {@link #getActiveLiveLanguages} method instead
  */
  getActiveLanguages(): Set<string>;
  /**
   * Returns a set of active site languages
   *
   * @return a set of active site languages
  */
  getActiveLiveLanguages(): Set<string>;
  /**
   * Returns an List of active site language  ( as Locale ).
   *
   * @return a List of Locale elements
  */
  getActiveLiveLanguagesAsLocales(): Locale[];
  /**
   * Returns an List of inactive site language  ( as Locale ).
   *
   * @return a List of Locale elements
  */
  getInactiveLanguagesAsLocales(): Locale[];
  getDefaultLanguage(): string;
  getDescr(): string;
  getHome(): JCRNodeWrapper;
  getLanguages(): Set<string>;
  /**
   * Returns an List of site language  ( as Locale ).
   *
   * @return an List of Locale elements.
  */
  getLanguagesAsLocales(): Locale[];
  getMandatoryLanguages(): Set<string>;
  getResolveSite(): JCRSiteNode;
  getServerName(): string;
  getServerNameAliases(): string[];
  getAllServerNames(): string[];
  getSiteKey(): string;
  getInstalledModules(): string[];
  /**
   * Returns a set of all installed modules for this site, their direct and transitive dependencies (the whole dependency tree).
   *
   * @return a set of all installed modules for this site, their direct and transitive dependencies (the whole dependency tree)
  */
  getInstalledModulesWithAllDependencies(): Set<string>;
  /**
   * Get installed modules with their dependencies
   * @return
  */
  getAllInstalledModules(): string[];
  /**
   * Returns the corresponding template set name of this virtual site.
   *
   * @return the corresponding template set name of this virtual site
  */
  getTemplatePackageName(): string;
  getTitle(): string;
  isHtmlMarkupFilteringEnabled(): boolean;
  isMixLanguagesActive(): boolean;
  isAllowsUnlistedLanguages(): boolean;
  isWCAGComplianceCheckEnabled(): boolean;
  /**
   * Returns true if this site is the default one on the server.
   *
   * @return true if this site is the default one on the server
  */
  isDefault(): boolean;
  /**
   * Returns a list holding the primary node type and all mixin node types for this node
   *
   * @return a list holding the primary node type and all mixin node types
   * @throws RepositoryException in case of JCR-related errors
  */
  getNodeTypes(): string[];
  /**
   * {@inheritDoc}
   *
   * @return The wrapped node at relPath.
  */
  getNode(s: string): JCRNodeWrapper;
  /**
   * Returns all child nodes of this node accessible through the current
   * Session. Does not include properties of this
   * Node. The same reacquisition semantics apply as with {@link
   * #getNode(String)}. If this node has no accessible child nodes, then an
   * empty iterator is returned.
   *
   * @return A NodeIterator over all child Nodes of
   *         this Node.
   * @throws RepositoryException if an error occurs.
  */
  getNodes(): JCRNodeIteratorWrapper;
  /**
   * Gets all child nodes of this node accessible through the current
   * Session that match namePattern. The pattern may
   * be a full name or a partial name with one or more wildcard characters
   * ("*"), or a disjunction (using the "|"
   * character to represent logical OR) of these. For example,
   * 
   * N.getNodes("jcr:* | myapp:report | my doc")
   * 
   * would return a NodeIterator holding all accessible child
   * nodes of N that are either called 'myapp:report',
   * begin with the prefix 'jcr:' or are called 'my
   * doc'.
   * 
   * The substrings within the pattern that are delimited by "|"
   * characters and which may contain wildcard characters ("*")
   * are called globs.
   * 
   * Note that leading and trailing whitespace around a glob is ignored, but
   * whitespace within a disjunct forms part of the pattern to be matched.
   * 
   * The pattern is matched against the names (not the paths) of the immediate
   * child nodes of this node.
   * 
   * If this node has no accessible matching child nodes, then an empty
   * iterator is returned.
   * 
   * The same reacquisition semantics apply as with {@link
   * #getNode(String)}.
   *
   * @param namePattern a name pattern.
   * @return a NodeIterator.
   * @throws RepositoryException if an unexpected error occurs.
  */
  getNodes(s: string): JCRNodeIteratorWrapper;
  /**
   * Gets all child nodes of this node accessible through the current
   * Session that match one or more of the nameGlob
   * strings in the passed array.
   * 
   * A glob may be a full name or a partial name with one or more wildcard
   * characters ("*"). For example,
   * 
   * N.getNodes(new String[] {"jcr:*", "myapp:report", "my
   * doc"})
   * 
   * would return a NodeIterator holding all accessible child
   * nodes of N that are either called 'myapp:report',
   * begin with the prefix 'jcr:' or are called 'my
   * doc'.
   * 
   * Note that unlike in the case of the {@link #getNodes(String)} leading and
   * trailing whitespace around a glob is not ignored.
   * 
   * The globs are matched against the names (not the paths) of the immediate
   * child nodes of this node.
   * 
   * If this node has no accessible matching child nodes, then an empty
   * iterator is returned.
   * 
   * The same reacquisition semantics apply as with {@link
   * #getNode(String)}.
   *
   * @param nameGlobs an array of globbing strings.
   * @return a NodeIterator.
   * @throws RepositoryException if an unexpected error occurs.
   * @since JCR 2.0
  */
  getNodes(nameGlobs: string[]): JCRNodeIteratorWrapper;
}

}
